<?php

namespace Drupal\quick_edit_variation\Form;

use Drupal\commerce_price\Price;
use Drupal\commerce_product\Entity\ProductVariationInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Ajax\PrependCommand;
use Drupal\Core\Ajax\RemoveCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Defines quick edit variation form.
 */
class QuickEditVariationForm extends FormBase {

  /**
   * Product variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariationInterface
   */
  protected $variation;

  /**
   * The form ID.
   *
   * @var string
   */
  protected $formId;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    if (empty($this->formId)) {
      $this->formId = 'quick_edit_variation';
      $variation = $this->variation;
      if ($variation) {
        $this->formId .= '_' . $variation->id();
      }
    }
    return $this->formId;
  }

  /**
   * Set form_id.
   *
   * @param string $form_id
   *   The form id.
   *
   * @return $this
   *   The form values.
   */
  public function setFormId(string $form_id) {
    $this->formId = $form_id;
    return $this;
  }

  /**
   * Set product variation.
   *
   * @param \Drupal\commerce_product\Entity\ProductVariationInterface $variation
   *   The product variation.
   *
   * @return $this
   *   The form values.
   */
  public function setVariation(ProductVariationInterface $variation) {
    $this->variation = $variation;
    return $this;
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $variation = $this->variation;
    $table_key = 'table_' . $variation->id();
    $form_fields['sku'] = [
      '#type' => 'textfield',
      '#title' => t('SKU'),
      '#default_value' => $variation->getSku(),
      '#maxlength' => 255,
      '#required' => TRUE,
    ];
    $list_price = $variation->getListPrice();
    $form_fields['list_price'] = [
      '#type' => 'container',
      'has_value' => [
        '#type' => 'checkbox',
        '#title' => t('Provide a list price'),
        '#default_value' => $list_price ? TRUE : FALSE,
      ],
      'value' => [
        '#type' => 'commerce_price',
        '#title' => t('List price'),
        '#available_currencies' => [],
        '#default_value' => $list_price ? $list_price->toArray() : NULL,
        '#allow_negative' => FALSE,
        '#states' => [
          "visible" => [
            ':input[name="' . $table_key . '[0][list_price][has_value]"]' => [
              "checked" => TRUE,
            ],
          ],
        ],
      ],
    ];
    $form_fields['price'] = [
      '#type' => 'commerce_price',
      '#title' => t('Price'),
      '#default_value' => $variation->getPrice()->toArray(),
      '#available_currencies' => [],
      '#allow_negative' => FALSE,
      '#required' => TRUE,
    ];
    $form_fields['status'] = [
      '#type' => 'checkbox',
      '#title' => t('Published'),
      '#available_currencies' => [],
      '#default_value' => $variation->isPublished(),
      '#allow_negative' => FALSE,
    ];
    $form_fields['save'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => t('Save'),
        '#ajax' => [
          'callback' => '::submitAjaxForm',
        ],
      ],
    ];
    $form[$table_key] = [
      '#type' => 'table',
      '#header' => [
        t('SKU'),
        t('List price'),
        t('Price'),
        t('Published'),
        t('Action'),
      ],
    ];
    $form[$table_key][] = $form_fields;
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * Submit ajax function.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The ajax response.
   */
  public function submitAjaxForm(array $form, FormStateInterface $form_state) {
    $variation = $this->variation;
    $variation_id = $variation->id();
    $table_key = 'table_' . $variation_id;
    $values = $form_state->getValues();
    $response = new AjaxResponse();
    $response->addCommand(new RemoveCommand('#result_message_' . $variation_id));
    $response->addCommand(new PrependCommand('.quick-edit-variation-' . $variation_id, '<div class="messages" id="result_message_' . $variation_id . '"></div>'));
    if ($values[$table_key][0]['sku'] && is_numeric($values[$table_key][0]['price']['number'])) {
      $variation->setSku($values[$table_key][0]['sku']);
      $variation->set('status', $values[$table_key][0]['status']);
      $variation->setPrice(new Price($values[$table_key][0]['price']['number'], $values[$table_key][0]['price']['currency_code']));
      if (is_numeric($values[$table_key][0]['list_price']['number'])) {
        $variation->setListPrice(new Price($values[$table_key][0]['list_price']['number'], $values[$table_key][0]['list_price']['currency_code']));
      }
      $variation->save();
      $response->addCommand(new MessageCommand($variation->getTitle() . ' Edited', '#result_message_' . $variation_id, ['type' => 'status']));
    }
    elseif (!$values[$table_key][0]['sku']) {
      $response->addCommand(new MessageCommand('SKU cannot be left blank.', '#result_message_' . $variation_id, ['type' => 'error']));
    }
    elseif (!$values[$table_key][0]['price']['number']) {
      $response->addCommand(new MessageCommand('Price cannot be left blank.', '#result_message_' . $variation_id, ['type' => 'error']));
    }
    elseif (!is_numeric($values[$table_key][0]['price']['number'])) {
      $response->addCommand(new MessageCommand('Price must be a number.', '#result_message_' . $variation_id, ['type' => 'error']));
    }
    return $response;
  }

}
